require_dependency "mdm/application_controller"

module Mdm
  class CommandsController < ApplicationController

    def index
      render json: { commands: case params[:filter]
                               when 'completed'
                                 Command.completed
                               when 'pending'
                                 Command.pending
                               else
                                Command.all
                               end }
    end

    def create
      @commands = create_commands(devices)
      @commands.each { |command|
        command.device.notify
      }
      render json: @commands
    rescue NoMethodError
      render json: { error: 'wrong command' }, status: :bad_request
    rescue ArgumentError
      render json: { error: 'bad arguments' }, status: :bad_request
    rescue ActiveRecord::RecordNotFound
      render nothing: true, status: :not_found
    end

    private

    def create_commands(devices)
      devices.map do |device|
        uuid, payload = create_command_payload_for_device(device)
        Command.create(
          uuid: uuid,
          payload: payload,
          status: Command::Status::IDLE,
          device: device
        )
      end
    end

    def create_command_payload_for_device(device)
      Payload.send(
        "payload_#{params[:command]}".to_sym,
        *[device, params]
      )
    end

    def devices
      @devices ||= if params[:devices]
                     Device.where(id: params[:devices])
                   else
                     Device.all
                   end
    end
  end
end
