module Mdm
  class ServerController < ApplicationController

    def server
      # Device is waiting for command
      if params[:Status] == Command::Status::IDLE
        return render nothing: true if device.commands.pending.empty?
        return render text: device.commands.pending.first.payload
      end

      # Device is updating command
      command.update(status: params[:Status])

      if command.complete?
        payload = Plist::parse_xml(command.payload).with_indifferent_access
        # Use data from `DeviceInformation` request to update the device locally
        if payload[:Command][:RequestType] == 'DeviceInformation'
          device.refresh!(params[:QueryResponses])
        end
      end
      
      render nothing: true
    end

    private

    def device
      @device ||= begin
                    device = Device.find_by_udid(params[:UDID])
                    return render nothing: true, status: :not_found unless device
                    device
                  end
    end

    def command
      @command ||= begin
                     command = Command.find_by_uuid(params[:CommandUUID])
                     return render nothing: true, status: :not_found unless command
                     command
                   end
    end
    
  end
end
