module Mdm

  module Enrollment

    class Client

      include HTTParty

      attr_reader :authenticated, :response

      base_uri 'https://mdmenrollment.apple.com'
      format :json
      headers 'X-Server-Protocol-Version' => '2',
              'Content-Type' => 'application/json;charset=UTF8'

      %w(get post put delete).each { |method|
        define_method(method) { |path, options = {}|
          inject_auth_token_if_authenticated(options)
          @response = self.class.send(method, path, options)
          Auth.update_token_from_response(@response)
          @response
        }
      }

      def authenticate!
        @authenticated = true
      end

      def extract_from_response(*attributes)
        return [] if response.nil?
        attributes.map { |attribute|
          response[attribute] }
      end

      private

      def inject_auth_token_if_authenticated(options = {})
        if authenticated
          (options[:headers] ||= {})
            .merge!('X-ADM-Auth-Session' => Auth.token)
        end
      end
    end

  end

end
