module Mdm

  module Enrollment

    class Auth < Service::Base

      cattr_accessor :consumer_key,
                     :consumer_secret,
                     :access_token,
                     :access_secret

      # Debug purpose only
      self.consumer_key    = "CK_974972d3a9acf95666e6c0cf4ae026f3d3ca6109b8582d95f3b7171f1f033749f66d8fa299202cf2cf616f7cbbc5fbb8"
      self.consumer_secret = "CS_f7a9a4bbcee4bfbf1e449a218bb74377c139f25e"
      self.access_token    = "AT_O8413116053O51d9691b9ff836ea72c731723efa2105710e1862O1438344338000"
      self.access_secret   = "AS_6eb94db58653f61ec3808f4051937f12e478948f"

      class << self

        def token
          @@token ||= new.obtain_token
        end

        def update_token_from_response(response)
          if response && token = response.headers['x-adm-auth-session']
            @@token = token
          end
        end

      end

      def obtain_token
        client.get '/session',
               headers: {
                 'Authorization' => authorization_header
               }
        client.response['auth_session_token']
      end

      def params
        @params ||= {
          oauth_consumer_key: consumer_key,
          oauth_token: access_token,
          oauth_nonce: nonce,
          oauth_timestamp: Time.now.getutc.to_i.to_s,
          oauth_version: '1.0',
          oauth_signature_method: 'HMAC-SHA1'
        }
      end

      private

      def authorization_header
        params_with_signature = params.merge(
          realm: 'MDM',
          oauth_signature: sign(base_str)
        )

        'OAuth ' + params_with_signature.map { |key, value|
          "#{key}=\"#{value}\""
        }.join(', ')
      end

      def base_str
        # Encode, sort and join params
        params_encoded = params.inject({}) { |hash, (key, value)|
          hash[CGI::escape(key.to_s)] = CGI::escape(value)
          hash
        }.sort.map { |key, value|
          "#{key}=#{value}"
        }.join('&')

        # Request method + url with path + params, escaped and joined by &
        [
          'GET',
          "https://mdmenrollment.apple.com/session",
          params_encoded
        ].map { |str|
          CGI::escape(str)
        }.join('&')
      end

      def sign(base_str)
        key = "#{CGI::escape(consumer_secret)}&#{CGI::escape(access_secret)}"
        digest = OpenSSL::Digest.new('sha1')
        hmac = OpenSSL::HMAC.digest(digest, key, base_str)
        CGI::escape(Base64.encode64(hmac).chomp.gsub(/\n/, ''))
      end

      def nonce
        @nonce ||= Base64.encode64(
          OpenSSL::Random.random_bytes(7)
        ).gsub(/\W/, '')
      end

    end

  end

end
