module Mdm

  module Enrollment

    module Service

      class Base

        def self.available_services
          [:account]
        end

        def self.accepted_params
          []
        end

        def start
          if authenticate?
            client.authenticate!
          end

          options = if [:post, :put].include? method
                      { body: params.to_json }
                    else
                      { query: params }
                    end

          client.send(method, path, options)
        end

        def result
          raise 'Subclasses must implement this method'
        end

        def method
          raise 'Subclasses must implement this method'
        end

        def path
          raise 'Subclasses must implement this method'
        end

        def authenticate?
          true
        end

        def params
          @params ||= {}
        end

        def params=(params)
          @params = params.permit(self.class.accepted_params)
        end

        protected

        def client
          @client ||= Enrollment::Client.new
        end

      end

    end

  end

end
