module Mdm

  module Enrollment

    class Devices < Service::Base

      attr_accessor :limit

      attr_reader :devices,
                  :cursor

      def self.accepted_params
        [:limit, :cursor]
      end

      def path
        '/server/devices'
      end

      def start
        params[:limit] = limit
        params[:cursor] = cursor.content

        super

        cursor.content, device_payloads, more_to_follow = client.extract_from_response(
          'cursor',
          'devices',
          'more_to_follow')

        create_or_update_devices(device_payloads)

        if cursor.save && more_to_follow
          start
        end
      end

      def method
        :post
      end

      def result
        {
          devices: devices,
          cursor: cursor
        }
      end

      private

      attr_accessor :devices

      def limit
        @limit ||= 1000
      end

      def devices
        @devices ||= []
      end

      def cursor
        @cursor ||= Cursor.find_or_create_by(
          service: Cursor::DEVICES)
      end

      def create_or_update_devices(device_payloads)
        devices.concat device_payloads.map { |device_payload|
          create_or_update_device(device_payload) }
      end

      def create_or_update_device(device_payload)
        Device.find_or_create_by(serial_number: device_payload['serial_number']) do |device|
          device.model = device_payload['model']
          device.description = device_payload['description']
          device.color = device_payload['color']
          device.asset_tag = device_payload['asset_tag']
          device.profile_status = device_payload['profile_status']
          device.profile_uuid = device_payload['profile_uuid']
          device.profile_assign_time = device_payload['profile_assign_time']
          device.profile_push_time = device_payload['profile_push_time']
          device.op_type = device_payload['op_type']
          device.op_date = device_payload['op_date']
          device.device_assigned_by = device_payload['device_assigned_by']
          device.device_assigned_date = device_payload['device_assigned_date']
        end
      end

    end

  end

end
