module Mdm
  class Payload

    def self.payload_device_lock(device, request_params = {})
      payload('DeviceLock')
    end

    def self.payload_device_info(device, request_params = {})
      payload('DeviceInformation', {
                Queries: [
                  "IsSupervised",
                  "AvailableDeviceCapacity",
                  "BluetoothMAC",
                  "BuildVersion",
                  "CarrierSettingsVersion",
                  "CurrentCarrierNetwork",
                  "CurrentMCC",
                  "CurrentMNC",
                  "DataRoamingEnabled",
                  "DeviceCapacity",
                  "DeviceName",
                  "ICCID",
                  "IMEI",
                  "IsRoaming",
                  "Model",
                  "ModelName",
                  "ModemFirmwareVersion",
                  "OSVersion",
                  "PhoneNumber",
                  "Product",
                  "ProductName",
                  "SIMCarrierNetwork",
                  "SIMMCC",
                  "SIMMNC",
                  "SerialNumber",
                  "UDID",
                  "WiFiMAC",
                  "UDID"
                ]
              })
    end

    def self.payload_install_app(device, request_params)
      params = request_params.permit(
        :Options,
        :ManifestURL,
        :ManagementFlags,
        :Configuration,
        :Attributes,
        :ChangeManagementState
      )
      
      payload('InstallApplication', params)
    end

    def self.payload_uninstall_app(device, request_params)
      params = request_params.permit(:Identifier)
      Rails.logger.info "params: #{params.inspect}"
      payload('RemoveApplication', params)
    end

    def self.payload_clear_password(device, request_params)
      params = request_params.permit(:device_id, :Where)
      device = Device.find(params[:device_id])        
      payload('ClearPasscode', params.merge({
                                              UnlockToken: StringIO.new(device.unlock_token)
                                            }))
    end

    def self.payload_voice_roaming(device, request_params)        
      payload('Settings', {
                Settings: [
                  {
                    Item: 'VoiceRoaming',
                    Enabled: request_params.required(:Enabled) == "true"
                  }
                ]
              })
    end

    def self.payload_data_roaming(device, request_params)        
      payload('Settings', {
                Settings: [
                  {
                    Item: 'DataRoaming',
                    Enabled: request_params.required(:Enabled) == "true"
                  }
                ]
              })
    end

    def self.payload_personal_hotspot(device, request_params)        
      payload('Settings', {
                Settings: [
                  {
                    Item: 'PersonalHotspot',
                    Enabled: request_params.required(:Enabled) == "true"
                  }
                ]
              })
    end

    def self.payload_set_wallpaper(device, request_params)
      payload('Settings', {
                Settings: [
                  {
                    Item: 'Wallpaper',
                    Image: StringIO.new(request_params.require(:Image).read),
                    Where: request_params[:Where]
                  }
                ]
              })
    end

    def self.payload_set_app_settings(device, request_params)
      payload('Settings', {
                Settings: [
                  {
                    Item: 'ApplicationConfiguration',
                    Identifier: request_params.require(:Identifier),
                    Configuration: request_params[:Configuration]
                  }
                ]
              })        
    end

    def self.payload_identify(device, request_params)
      params = request_params.permit(:Identifier)
      payload_set_app_settings(
        device,
        ActionController::Parameters.new(
          {
            Configuration: {
              device: {
                udid: device.udid,
                serial_number: device.serial_number || 'N/A'
              }
            }
          }.merge(params)
        )
      )
    end

    private

    def self.payload(command, params = {})
      uuid = SecureRandom.uuid

      [
        uuid,
        {
          CommandUUID: uuid,
          Command: { RequestType: command }.merge(params)
        }.to_plist.gsub("\n", '').gsub("\t", '')
      ]
    end
  end
end
